"use strict";

{
	C3.Plugins.RGBZ_JsonManager.Acts =
	{
		LoadJsonManager (enableCaching)
		{
			let self = this;

			let files;
			let slots;
			let numOfFiles;
			
			if (self.rgbzJsonFiles.trim()=='' || self.rgbzJsonSlots.trim()==''){
				logMessage("Files and Slots not specified.");
				return;
			}
			else {
				files = self.rgbzJsonFiles.split(',');
				slots = self.rgbzJsonSlots.split(',');
	
				if (files.length != slots.length){
					logMessage('Uneven number of files and slots.');
					return;
				}
				else {
					numOfFiles = files.length;
				}
			}
	
			try {
				if (enableCaching.toLowerCase()=='false') {
					clearStore();
				}
	
				for (let i=0;i<files.length;i++) {
					let index = i;
					let xhttp = new XMLHttpRequest();
					let slotName = slots[index].trim().toLowerCase();
	
					let cacheCheck = getItem(slotName);
		
					if (!cacheCheck) {
						xhttp.onreadystatechange = function () {
							if (this.readyState == 4 && this.status == 200) {
								numOfFiles--;
	
								self.jsonSlots[slotName] = JSON.parse(xhttp.responseText);
								storeItem(slotName, self.jsonSlots[slotName]);
	
								if (numOfFiles<=0){
									//All files have been loaded, trigger complete event
									self.Trigger(C3.Plugins.RGBZ_JsonManager.Cnds.JsonManagerLoaded);
								}
							}
							else if (this.readyState == 4) {
								//An error has been returned from the server
								logMessage('Json ajax file retrieve falied - ' + this.status.toString() + ' - ' + this.statusText);
							}
						};
	
						let pathToFile = (files[i].trim().toLowerCase().startsWith('http'))?files[i].trim().toLowerCase():window.cr_previewProjectFiles[files[i].trim().toLowerCase()];
						xhttp.open("GET", pathToFile, true);
						xhttp.send();
					}
					else {
						self.jsonSlots[slotName] = cacheCheck;
						numOfFiles--;
	
						if (numOfFiles<=0){
							//All files have been loaded, trigger complete event
							self.Trigger(C3.Plugins.RGBZ_JsonManager.Cnds.JsonManagerLoaded);
						}
					}
				}
			}
			catch(err){
				logMessage(err.message);
			}
		},
		LoadJsonManagerFromString (slotName, jsonString, enableCaching)
		{
			let self = this;

			try {
				if (enableCaching.toLowerCase()=='false') {
					clearStore();
				}
	
				self.jsonSlots[slotName] = JSON.parse(jsonString);
				storeItem(slotName, self.jsonSlots[slotName]);
	
				//Theoretically a trigger isn't required since this a synchronous operation, but I
				//will fire it anyway in case a user has logic down the line that relies on a loaded trigger occurring.
	
				self.Trigger(C3.Plugins.RGBZ_JsonManager.Cnds.JsonManagerLoaded);
			}
			catch(err){
				console.log(err.message);
			}
		},
		SetJsonValue (slot, query, property, val, withinExistingLoop)
		{
			let foundObj = (withinExistingLoop)?getJsonObjectWorker(this.lastJsonObjectSelectedStack.peek(),query):getJsonObject(this.jsonSlots, slot, query);

			if (foundObj && property && typeof(foundObj[property])!=='undefined'){
				foundObj[property] = val;
	
				storeItem(slot, this.jsonSlots[slot]);
			}
			else {
				logMessage('Query could not locate object to perform a set on.');
			}
		},
		SetJsonSubToNewSlot (oldSlot, query, newSlot, withinExistingLoop)
		{
			let foundObj = (withinExistingLoop)?getJsonObjectWorker(this.lastJsonObjectSelectedStack.peek(),query):getJsonObject(this.jsonSlots, oldSlot, query);

			if (foundObj) {
				this.jsonSlots[newSlot] = JSON.parse(JSON.stringify(foundObj));
	
				storeItem(newSlot, this.jsonSlots[newSlot]);
			}
			else {
				logMessage("Query to locate Json to put into a new slot can't be found.");
			}
		},
		ResetJsonSlotFromBackup (slot)
		{
			let restoredItem = resetKeyFromBak(slot);

			if (restoredItem) {
				this.jsonSlots[slot] = restoredItem;
			}
		},
		RemoveJsonSlot (slot)
		{
			removeItem(slot);

			delete this.jsonSlots[slot];
		}
	};
}